## Bitmap image class
## Copyright Carl Pearson April 2012
## April 2012
## Compatible Python 3.2.x

## import the struct module to use C structs to store the image

import struct
import random

class BMP:
    def __init__(self,name):
        ## create the dictionary defines the bitmap
        ## Windows version 3 DIB header
        self.name = name
        self.header = {
            'mn1':66,
            'mn2':77,
            'filesize':0,
            'undef1':0,
            'undef2':0,
            'offset':54,
            'headerlength':40,
            'width':3,
            'height':3,
            'colorplanes':0,
            'colordepth':24,
            'compression':0,
            'imagesize':0,
            'res_hor':0,
            'res_vert':0,
            'palette':0,
            'importantcolors':0
            }
        self.bytes = bytes()
        self.genEmptyBMP()

    def genEmptyBMP(self):
        ## the length of a row
        row_length = int(self.header['width']*self.header['colordepth']/8)

        ## rows must be a multiple of 4 long
        row_mod = int(row_length % 4)
        if row_mod == 0:
            padding = 0
        else:
            padding = (4 - row_mod)
        row_length += padding

        ## the full length of an image
        image_length = row_length * self.header['height']

        self.bytes = bytearray(image_length)

    def setPixel(self,x,y,r,g,b):
        ## the trick here is finding the right pixel index
        ## x is the column from the left, which is pretty easy
        byte_column = 3 * x ## 3 bytes per pixel
        ## y is the row from the top, but the first bytes of
        ## self.bytes are actually the bottom row
        ## length of row in bytes
        row_length = int(self.header['width']*self.header['colordepth']/8)
        ## rows must be a multiple of 4 bytes long
        ## its part of the spec
        row_mod = int(row_length % 4)
        if row_mod == 0:
            padding = 0
        else:
            padding = (4 - row_mod)
        row_length += padding
        nrows = self.header['height']
        ## calculate the index of the first byte that makes up the pixel
        index = (nrows - 1 -y) * row_length + byte_column
        self.bytes[index    ] = b
        self.bytes[index + 1] = g
        self.bytes[index + 2] = r

    def writeBMP(self):
        d = self.header
        ## get all the header info for the BMP format
        mn1 = struct.pack('<B',d['mn1'])
        mn2 = struct.pack('<B',d['mn2'])
        filesize = struct.pack('<L',d['filesize'])
        undef1 = struct.pack('<H',d['undef1'])
        undef2 = struct.pack('<H',d['undef2'])
        offset = struct.pack('<L',d['offset'])
        headerlength = struct.pack('<L',d['headerlength'])
        width = struct.pack('<L',d['width'])
        height = struct.pack('<L',d['height'])
        colorplanes = struct.pack('<H',d['colorplanes'])
        colordepth = struct.pack('<H',d['colordepth'])
        compression = struct.pack('<L',d['compression'])
        imagesize = struct.pack('<L',d['imagesize'])
        res_hor = struct.pack('<L',d['res_hor'])
        res_vert = struct.pack('<L',d['res_vert'])
        palette = struct.pack('<L',d['palette'])
        importantcolors = struct.pack('<L',d['importantcolors'])
        ## create the outfile
        outfile = open(self.name,'wb')
        ## write the header + the bytes
        outfile.write(mn1+mn2+filesize+undef1+undef2+offset+headerlength+width+height+\
                  colorplanes+colordepth+compression+imagesize+res_hor+res_vert+\
                  palette+importantcolors+self.bytes)
        ## close the output file
        outfile.close()

    def setWidth(self,n):
        self.header['width'] = n

    def getWidth(self):
        return self.header['width']

    def setHeight(self, n):
        self.header['height'] = n

    def getHeight(self):
        return self.header['height']

    def setSize(self, x, y):
        self.setWidth(x)
        self.setHeight(y)
        self.genEmptyBMP()

    def getSize(self):
        return (self.getWidth(), self.getHeight())

###################################    
def main():
    def rand_color():
        x = random.randint(0,255)
        return x
    
    s = BMP("test.bmp")
    s.setWidth(128)
    s.setHeight(128)
    s.genEmptyBMP()

    for x in range(0, s.getWidth()):
        for y in range(0, s.getHeight()):
            r = rand_color()
            g = rand_color()
            b = rand_color()
            s.setPixel(x, y, r, g, b)
    
    s.writeBMP()
###################################    

if __name__ == '__main__':
    main()